#usage nm-pcrel.rb ../../src/kegg/xml/kgml/non-metabolic/ ../out/non-metabolic_cpd-ko.txt

require 'rexml/document'

mapIDs = ['02010','02060','02020']

class Ko
    attr_reader :id, :name
    def initialize(id,name)
        @id = id; @name = name;
    end
end
koList = {}

class Compound
    attr_reader :id, :name
    def initialize(id,name)
        @id = id; @name = name;
    end
end
compoundList = {}

groupList = {}

relationList = []

mapIDs.each do |map|
  fp = File.open(ARGV[0]+'ko/ko'+map+'.xml')
    doc = REXML::Document.new fp
    doc.elements.each('pathway/entry') do |entry|
      id = "#{map}__#{entry.attributes['id']}"
      if entry.attributes['type'] == 'ortholog'
        names = entry.attributes['name'].split("\s")
        name = names.map{|ko| ko.gsub(/^ko\:/, '')}.join('+')
        koList[id] = Ko.new(id, name)
      elsif entry.attributes['type'] == 'compound'
        names = entry.attributes['name'].split("\s")
        name = names.map{|cpd| cpd.gsub(/^cpd\:|^gl\:|^dr\:/, '')}.join('+')
        compoundList[id] = Compound.new(id, name)
      elsif entry.attributes['type'] == 'group'
        groupList[id] = []
        entry.elements.each('component') do |component|
          groupList[id].push("#{map}__#{component.attributes['id']}")
        end
      end
    end
    doc.elements.each('pathway/relation') do |relation|
      if relation.attributes['type'] == 'PCrel'
        id1 = "#{map}__#{relation.attributes['entry1']}"
        id2 = "#{map}__#{relation.attributes['entry2']}"
        relationList.push([id1, id2])
      end
    end
  fp.close
end

pairs = {}
relationList.each do |ids|
  id1 = groupList[ids[0]].nil? ? [ids[0]] : groupList[ids[0]]
  id2 = groupList[ids[1]].nil? ? [ids[1]] : groupList[ids[1]]
  kos1 = []
  compounds1 = []
  kos2 = []
  compounds2 = []
  id1.each do |id|
    kos1.concat(koList[id].name.split('+')) unless koList[id].nil?
    compounds1.concat(compoundList[id].name.split('+')) unless compoundList[id].nil?
  end
  id2.each do |id|
    kos2.concat(koList[id].name.split('+')) unless koList[id].nil?
    compounds2.concat(compoundList[id].name.split('+')) unless compoundList[id].nil?
  end
  if kos2.length > 0 && compounds1.length > 0
    compounds1.each do |cpd|
      pairs[cpd] = [] if pairs[cpd].nil?
      pairs[cpd].push(kos2.join('+'))
    end
  end
end

w = open(ARGV[1],'w')
w.flock(File::LOCK_EX)
pairs.each do |k,v|
  w.puts("#{k}\t#{v.join(',')}")
end
w.flock(File::LOCK_UN)
w.close
